--------------------------------------------------------------------------------
-- Shockwave Rocket by holzchopf
--------------------------------------------------------------------------------

-- setup
if holzchopf==nil then holzchopf={} end
holzchopf.shockwaverocket={}
holzchopf.shockwaverocket.rocket={}

-- load ressources
holzchopf.shockwaverocket.gfx_wpn=loadgfx("weapons/launcher.bmp")
setmidhandle(holzchopf.shockwaverocket.gfx_wpn)
holzchopf.shockwaverocket.gfx_proj=loadgfx("holzchopf/shockwaverocket.bmp")
setmidhandle(holzchopf.shockwaverocket.gfx_proj)
holzchopf.shockwaverocket.sfx_shot=loadsfx("rocketrelease.wav")
holzchopf.shockwaverocket.sfx_blast=loadsfx("holzchopf/shockwaveblast.ogg")

--------------------------------------------------------------------------------
-- Weapon
--------------------------------------------------------------------------------
holzchopf.shockwaverocket.id=addweapon("holzchopf.shockwaverocket","Shockwave Rocket",holzchopf.shockwaverocket.gfx_proj,1)

function holzchopf.shockwaverocket.draw()
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,233,110)
	drawinhand(holzchopf.shockwaverocket.gfx_wpn,6,0)
	-- HUD Crosshair
	if weapon_shots==0 then
		hudcrosshair(6,3)
	end
end

function holzchopf.shockwaverocket.attack(attack)
	if (weapon_shots<=0) and (attack==1) then
		-- no more weapon switching!
		useweapon(0)
		playsound(holzchopf.shockwaverocket.sfx_shot)
		weapon_shots=weapon_shots+1
		id=createprojectile(holzchopf.shockwaverocket.rocket.id)
		projectiles[id]={}
		-- ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- set initial position of projectile
		projectiles[id].x=getplayerx(0)+(6*getplayerdirection(0))+math.sin(math.rad(getplayerrotation(0)))*10.0
		projectiles[id].y=getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*10.0
		-- initial movement
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*20
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*20
		projectiles[id].x=projectiles[id].x-projectiles[id].sx
		projectiles[id].y=projectiles[id].y-projectiles[id].sy
		holzchopf.shockwaverocket.rocket.move(id,0)
		-- set speed of projectile
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*15.0
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*15.0
		-- fx
		recoil(5)
		endturn()
	end
end

--------------------------------------------------------------------------------
-- Rocket & Shockwave
--------------------------------------------------------------------------------
holzchopf.shockwaverocket.rocket.id=addprojectile("holzchopf.shockwaverocket.rocket")

function holzchopf.shockwaverocket.rocket.draw(id)
	-- draw projectile
	setblend(blend_alpha)
	setcolor(255,255,255)
	setalpha(1)
	setscale(1,1)
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	drawimage(holzchopf.shockwaverocket.gfx_proj,projectiles[id].x,projectiles[id].y)
	-- draw arrow if out of screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function holzchopf.shockwaverocket.rocket.update(id)
	holzchopf.shockwaverocket.rocket.move(id,1)
end

function holzchopf.shockwaverocket.rocket.move(id,fx)
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- particle tail
	if (fx==1) then
		particle(p_smoke,projectiles[id].x-math.sin(math.rad(rot))*7,projectiles[id].y+math.cos(math.rad(rot))*7)
		particlespeed(math.random(-2,2)*0.1,math.random(-2,2)*0.1)
		particlefadealpha(0.01)
		particle(p_lightpuff,projectiles[id].x-math.sin(math.rad(rot))*6,projectiles[id].y+math.cos(math.rad(rot))*6)
		particlefadealpha(0.04)
	end
	-- wind + gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()*0.1
	projectiles[id].sy=projectiles[id].sy+getgravity()*1.5
	-- move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby
		-- collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*3,projectiles[id].y-math.cos(math.rad(rot))*3)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				-- destroy terrain
				terraincircle(projectiles[id].x,projectiles[id].y,20,0x00000000)
				-- crater
				grey=math.random(0,40)
				if math.random(0,1)==1 then
					terrainalphaimage(gfx_crater75,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
				else
					terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
				end
				-- push players
				players=playertable(0,0)
				for i=1,#players do
					player=players[i]
					-- player in range
					local dx=getplayerx(player)-projectiles[id].x
					local dy=getplayery(player)-projectiles[id].y
					local sx,sy=holzchopf.shockwaverocket.rocket.shockspeed(dx,dy)
					if sx~=0 or sy~=0 then
						playerpush(player,sx,sy,1)
					end
				end
				-- push objects
				obs=objecttable()
				for i=1,#obs do
					object=obs[i]
					-- object in range
					local dx=getobjectx(object)-projectiles[id].x
					local dy=getobjecty(object)-projectiles[id].y
					local sx,sy=holzchopf.shockwaverocket.rocket.shockspeed(dx,dy)
					if sx~=0 or sy~=0 then
						objectpush(object,sx,sy,1,1)
					end
				end
				-- explosion
				particle(p_muzzle,projectiles[id].x,projectiles[id].y)
				particlecolor(255,255,255)
				for i=0,3 do
					particle(p_explosion,projectiles[id].x,projectiles[id].y)
					local s=math.random(5,10)*0.1
					particlesize(s,s)
					particlecolor(255,255-math.random(0,128),0)
					particlespeed(math.random(-10,10)*0.1,math.random(-10,10)*0.1)
				end
				quake(20)
				-- create shockwave
				particle(p_ring,projectiles[id].x,projectiles[id].y)
				particlesize(0.05,0.05)
				particlespeed(1.2,1.2)
				playsound(holzchopf.shockwaverocket.sfx_blast)
				-- free projectile
				freeprojectile(id)
				break
			end
		else
			projectiles[id].ignore=0
		end
		-- water
		if (projectiles[id].y)>getwatery()+5 then
			-- fx
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- free projectile
			freeprojectile(id)
			break
		end
	end
	-- scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end

function holzchopf.shockwaverocket.rocket.shockspeed(dx,dy)
	local dist=math.sqrt(dx*dx+dy*dy)
	if dist<=200 and dist>0 then
		local f=10*(1-dist*0.005)/dist
		local sx=dx*f
		local sy=dy*f
		return sx,sy
	end
	return 0,0
end